package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.MenuProvider;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.AABB;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.energy.CapabilityEnergy;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage;
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.math.BigDecimal;
import java.util.List;

public class BlockEntityAndroidStation extends BlockEntity implements MenuProvider {
	@Override
	public Component getDisplayName() {
		return new TranslatableComponent("container.otm.android_station");
	}

	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new AndroidStationMenu(containerID, inventory, new AndroidStationMenu.AndroidStationContainer(ply), this);
	}

	private final MatteryMachineEnergyStorage energy = new MatteryMachineEnergyStorage(MatteryMachineEnergyStorage.MachineType.WORKER, new BigDecimal(1_000_000_000_000L));

	public BlockEntityAndroidStation(BlockPos p_155229_, BlockState p_155230_) {
		super(OverdriveThatMatters.ANDROID_STATION_FACTORY, p_155229_, p_155230_);
	}

	@Nonnull
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (cap == MatteryCapability.ENERGY || cap == CapabilityEnergy.ENERGY) {
			return LazyOptional.of(() -> energy).cast();
		}

		return super.getCapability(cap, side);
	}

	@Override
	public CompoundTag save(CompoundTag nbt) {
			nbt.put("energy_cap", energy.serializeNBT());
		return super.save(nbt);
	}

	public void load(CompoundTag nbt) {
		super.load(nbt);

		if (nbt.contains("energy_cap") && nbt.get("energy_cap") instanceof CompoundTag tag)
			energy.deserializeNBT(tag);
	}

	void tick() {
		if (!hasLevel())
			return;

		if (energy.getEnergyStored() <= 0)
			return;

		BlockPos pos = getBlockPos();
		List<LivingEntity> entities = getLevel().getEntitiesOfClass(LivingEntity.class, new AABB(pos.getX(), pos.getY(), pos.getZ(), pos.getX() + 1, pos.getY() + 2, pos.getZ() + 1));

		for (LivingEntity ent : entities) {
			LazyOptional<IAndroidCapability> resolver = ent.getCapability(MatteryCapability.ANDROID);

			if (resolver.isPresent()) {
				IAndroidCapability capability = resolver.resolve().get();

				BigDecimal missing = capability.getMissingPower();

				if (missing.compareTo(BigDecimal.ZERO) == 1) {
					BigDecimal extract = energy.extractEnergyInner(missing, true);

					if (extract.compareTo(BigDecimal.ZERO) == 1) {
						BigDecimal received = capability.receiveEnergyOuter(extract, true);

						if (received.compareTo(BigDecimal.ZERO) == 1) {
							energy.extractEnergyInner(extract, false);
							capability.receiveEnergyOuter(extract, false);
						}
					}
				}
			}
		}
	}

	public static <T extends BlockEntity> void tick(Level level, BlockPos blockPos, BlockState blockState, T t) {
		if (t instanceof BlockEntityAndroidStation tile) {
			tile.tick();
		}
	}
}
