package ru.dbotthepony.mc.otm.menu;

import net.minecraft.core.BlockPos;
import net.minecraft.world.Container;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.inventory.Slot;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.energy.CapabilityEnergy;
import net.minecraftforge.energy.IEnergyStorage;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityAndroidStation;
import ru.dbotthepony.mc.otm.capability.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.data.BigDecimalDataContainer;

import javax.annotation.Nullable;

public class AndroidStationMenu extends AbstractContainerMenu {
	public static class AndroidStationContainer extends SimpleContainer {
		public final Player player;
		private final IAndroidCapability android;

		public AndroidStationContainer(Player ply) {
			super(1);
			player = ply;

			LazyOptional<IAndroidCapability> capability = ply.getCapability(MatteryCapability.ANDROID);
			android = capability.isPresent() ? capability.resolve().get() : null;

			if (android != null) {
				setItem(0, android.getBatteryItemStack());
			}
		}
	}

	public static class AndroidBatterySlot extends Slot {
		private final IAndroidCapability android;

		public AndroidBatterySlot(AndroidStationContainer container, int index, int x, int y) {
			super(container, index, x, y);
			android = container.android;
		}

		@Override
		public boolean mayPlace(ItemStack stack) {
			LazyOptional<IEnergyStorage> capability = stack.getCapability(CapabilityEnergy.ENERGY);
			return capability.isPresent() && capability.resolve().get().canExtract();
		}

		@Override
		public void set(ItemStack stack) {
			super.set(stack);
			android.setBatteryItemStack(stack);
		}
	}

	private BlockEntityAndroidStation tile;
	private Inventory inventory;
	private Player ply;
	private AndroidStationContainer container;
	public BigDecimalDataContainer energy;
	public BigDecimalDataContainer max_energy;

	// Clientside
	public AndroidStationMenu(int containerID, Inventory inventory) {
		this(containerID, inventory, new AndroidStationContainer(inventory.player), null);
	}

	// Serverside
	public AndroidStationMenu(int containerID, Inventory inventory, AndroidStationContainer container, @Nullable BlockEntityAndroidStation tile) {
		super(OverdriveThatMatters.ANDROID_STATION_MENU_TYPE, containerID);
		this.inventory = inventory;
		this.container = container;
		this.tile = tile;
		this.ply = inventory.player;

		if (tile == null) {
			energy = new BigDecimalDataContainer();
			max_energy = new BigDecimalDataContainer();
		} else {
			energy = new BigDecimalDataContainer() {
				@Override
				protected void updateValue() {
					LazyOptional<IMatteryEnergyStorage> capability = tile.getCapability(MatteryCapability.ENERGY);

					if (capability.isPresent())
						setDecimal(capability.resolve().get().getBatteryLevel());
				}
			};

			max_energy = new BigDecimalDataContainer() {
				@Override
				protected void updateValue() {
					LazyOptional<IMatteryEnergyStorage> capability = tile.getCapability(MatteryCapability.ENERGY);

					if (capability.isPresent())
						setDecimal(capability.resolve().get().getMaxBatteryLevel());
				}
			};
		}

		this.addSlot(new AndroidBatterySlot(this.container, 0, 39, 15));

		for (int i = 0; i < 3; ++i) {
			for (int j = 0; j < 9; ++j) {
				this.addSlot(new Slot(inventory, j + i * 9 + 9, 8 + j * 18, 97 + i * 18));
			}
		}

		for (int k = 0; k < 9; ++k) {
			this.addSlot(new Slot(inventory, k, 8 + k * 18, 155));
		}

		this.addDataSlots(energy);
		this.addDataSlots(max_energy);
	}

	@Override
	public boolean stillValid(Player player) {
		if (tile == null)
			return true;

		if (player.level.getBlockEntity(tile.getBlockPos()) != tile) {
			return false;
		}

		BlockPos pos = tile.getBlockPos();
		return player.distanceToSqr((double)pos.getX() + 0.5D, (double)pos.getY() + 0.5D, (double)pos.getZ() + 0.5D) <= 64.0D;
	}
}
